<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://www.emojics.com
 * @since      1.0.0
 *
 * @package    Emojics_WP
 * @subpackage Emojics_WP/admin
 * @author     Stefano <stefano.marra1987@gmail.com>
 */
class Emojics_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name . '-admin-css', plugin_dir_url( __FILE__ ) . 'css/min/emojics-admin.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_register_script( $this->plugin_name . '-admin-js', plugin_dir_url( __FILE__ ) . 'js/dist/emojics-admin.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name . '-admin-js' , 'emojics_data', array(
			'ajax_url' => admin_url( 'admin-ajax.php' )
		));
		wp_enqueue_script( $this->plugin_name . '-admin-js' );
	}

	/**
	 * Check if API Key response is valid
	 *
	 * @since 1.0.0
	 */
	public function is_authentication_valid($auth_response = null) {
		if ( is_null($auth_response) ) return false;

		if ( !is_array($auth_response) ) {
			return false;
		}

		if ( !isset($auth_response['id']) && !isset($auth_response['js_code_snippet']) ) {
			return false;
		}

		/**
		 * Check host
		 */
		$host = parse_url(home_url(), PHP_URL_HOST);
		if ( preg_replace('/^www\./','', $host) != preg_replace('/^www\./','', strtolower($auth_response['host'])) ) {
			return false;
		}

		return true;
	}

	/**
	 * Get authentication response from emojics rest api
	 *
	 * @since 	1.0.0
	 */
	public function authenticate_api_key($api_key = null) {
		if ( is_null($api_key) ) return false;

		$args = array(
			'headers' => array(
				'Authorization' => 'Basic ' . base64_encode( $api_key )
			)
		);

		return json_decode( wp_remote_retrieve_body( wp_remote_request(EMJCS_API_ENDPOINT . '/widget', $args) ), true );
	}

	/**
	 * Authenticates API key on option update
	 */
	public function authenticate_api_key_on_option_update($old_api_key, $api_key, $option) {
		if ( !$api_key ) return null;


		$res = $this->authenticate_api_key( $api_key );

		if ( $this->is_authentication_valid($res) ) {

			/**
			 * Valid API Key
			 */
			emojics_set_option( 'js_code', $res['js_code_snippet'] );
			return true;
		}

		/**
		 * API Key Not Valid
		 */
		emojics_set_option( 'js_code', null );
		add_settings_error( 'invalid-api-key', esc_attr( 'settings_updated' ), 'Invalid API Key', 'error' );

		return false;
	}

	/**
	 * Ajax method that handles authentication of the api key
	 *
	 * @since	1.0.0
	 */
	public function authenticate_api_key_ajax() {
		$api_key = $_POST['emojics_api_key'];

		$result = [
			'success' => false
		];

		$res = $this->authenticate_api_key( $api_key );

		if ( $this->is_authentication_valid($res) ) {
			emojics_set_option( 'api_key', $api_key );
			emojics_set_option( 'js_code', $res['js_code_snippet'] );

			$result['success'] = true;
		}

		echo json_encode( $result );
		die();
	}

	/**
	 * Get registration response from emojics rest api
	 *
	 * @since 1.6.0
	 */
	private function register($data = null) {
		if ( !$data ) return false;

		$args = array(
			'method'  => 'PUT',
			'headers' => array(
				'Content-Type'    => 'application/json'
			),
			'body' => json_encode($data)
		);

		$request = wp_remote_request(EMJCS_API_ENDPOINT . '/widget', $args);
		return json_decode( wp_remote_retrieve_body( $request ), true );
	}

	/**
	 * Check if registration response is valid
	 *
	 * @since 1.6.0
	 */
	private function is_registration_valid($registration_response = null) {
		if ( !$registration_response ) return false;

		if ( !is_array($registration_response) ) {
			return false;
		}

		if ( !isset($registration_response['api_key']) || !$registration_response['api_key'] ) {
			return false;
		}

		if ( !isset($registration_response['js_code_snippet']) || !$registration_response['js_code_snippet'] ) {
			return false;
		}

		return true;
	}

	/**
	 * Ajax method that handles registration through the emojics api
	 *
	 * @since	1.6.0
	 */
	public function registration_ajax() {
		$registration_data = [
			'first_name' => $_POST['first_name'],
			'last_name'  => $_POST['last_name'],
			'site'       => $_POST['site'],
			'email'      => $_POST['email'],
			'password'   => $_POST['password']
		];

		$result = [
			'success' => false
		];

		$res = $this->register( $registration_data );

		if ( $this->is_registration_valid($res) ) {
			emojics_set_option( 'api_key', $res['api_key'] );
			emojics_set_option( 'js_code', $res['js_code_snippet'] );

			$result['success'] = true;
		}
		else {
			$result = array_merge($result, $res);
		}

		echo json_encode( $result );
		die();
	}

	/**
	 * Show some admin messages
	 */
	public function admin_messages() {

		if ( !emojics_get_option('api_key', false) ) {
			return;
		}

		$install_date = emojics_get_option('install_date', false);
		$next_favor_date = emojics_get_option('next_review_favor', false);
		$user_rated = emojics_get_option('user_rated', 'no');

		if ( !$install_date ) {
			$now = date( 'Y-m-d h:i:s' );
			emojics_set_option( 'install_date', $now );
			emojics_set_option( 'next_review_favor', date('Y-m-d h:i:s', strtotime("$now +1 day")) );
    		emojics_set_option( 'user_rated', 'no' );
		}

		if ( $next_favor_date == 'no' ) {
			return;
		}

		$now_time = new DateTime( date( 'Y-m-d h:i:s' ) );

		$next_favor_time = new DateTime( $next_favor_date );
		$install_time = new DateTime( $install_date );
		$diff_interval = round( ($next_favor_time->format( 'U' ) - $now_time->format( 'U' )) / (60 * 60 * 24) );

		/**
		 * If user has already rated the plugin return
		 */
		if ( $user_rated == 'yes' ) {
			return;
		}

		/**
		 * If it's still not time to display the favor...
		 */
		if ( $diff_interval > 0 ) {
			return;
		}

		$logo = '<img style="width: 44px; display: block; margin-bottom: 10px;" src="' .  EMJCS_URL . 'admin/img/emojics-logo-icon.svg' . '" />';

		$title = '<h3>Special Deal Offer</h3>';

		$reward_message = '
			<div class="emojics-favor-message__reward">
				<div class="review">
					<small>Give Us</small>
					<img  src="' .  EMJCS_URL . 'admin/img/stars.png' . '" />
					<small>5 Stars Review</small>
				</div>
				<div class="reward">
					<small>You Get</small>
					<strong>1 Month Free</strong>
					<small>100% Off Coupon Code</small>
				</div>
			</div>
		';

		/**
		 * Defaut message,
		 */
		$message = '
			Hey, you\'ve been using <strong>Emojics</strong> for a couple of weeks – that’s awesome! Could you please do me a BIG favor and give it a 5-star rating on WordPress?<br>
			I will reward you with a <strong>100% OFF Coupon Code to access our PRO Plan for 1 month for free</strong>.<br>
			Just to help us spread the word and boost our motivation.<br><br>
			Thank you very much,<br> ~Fabiola Farina
		';

		$days = round( ($now_time->format( 'U' ) - $install_time->format( 'U' )) / (60 * 60 * 24) );

		/**
		 * After 3 weeks from installation, don't annoy the user anymore
		 */
		if ( $days > 21 ) {
			return;
		}

		/**
		 * User has installed the plugin within 6 days ago
		 */
		if ( $days < 7 ) {
			$message = '
				Awesome! Thanks for giving me the chance to show you what <strong>Emojics</strong> is capable of... I know you will love it!<br>
				Could you please do me a BIG favor and give it a 5-star rating on WordPress? I will reward you with a <strong>100% OFF Coupon Code to access our PRO Plan for 1 month for free</strong>.<br>
				Just to help us spread the word and boost our motivation.<br><br>
				Again, thank you very much,<br> ~Fabiola Farina
			';
		}
		else if ( $days < 14 ) {
			$message = '
				Hey, you\'ve been using <strong>Emojics</strong> for more than 1 week – that’s awesome! Could you please do me a BIG favor and give it a 5-star rating on WordPress?<br>
				I will reward you with a <strong>100% OFF Coupon Code to access our PRO Plan for 1 month for free</strong>.<br>
				Just to help us spread the word and boost our motivation.<br><br>
				Thank you very much,<br> ~Fabiola Farina
			';
		}

		echo '
			<div class="emojics-favor-message update-nag">
				<div class="emojics-favor-message__content">
			    	' . $logo . $title . $reward_message . $message . '
			        <ul>
			            <li><a class="emojics-favor-message__ok" href="https://wordpress.org/support/plugin/emojics-wp/reviews/?filter=5#new-post" target="_new" title=Yes, Emojics Increased My User Engagement">Ok, you deserve it</a></li>
			            <li><a class="emojics-favor-message__already" href="javascript:void(0);" title="I already did" style="">I already did</a></li>
			            <li><a class="emojics-favor-message__later" href="javascript:void(0);" title="Nope, maybe later">Nope, maybe later</a></li>
			        </ul>
			        <a class="emojics-favor-message__close" href="javascript:void(0);" title="Close">✕</a>
			    </div>
		    </div>
	    ';
	}

	/**
	 * Ajax method that handles favor response
	 *
	 * @since	1.6.0
	 */
	public function handle_favor_response_ajax() {
		$user_response = $_POST['response'];

		$result = [
			'success' => false
		];

		switch ($user_response) {

			/**
			 * The user response: Yes, you deserve it
			 * Set next_favor_date to "no" and user_rated to "yes"
			 */
			case 'ok':
				emojics_set_option( 'next_review_favor', 'no' );
				emojics_set_option( 'user_rated', 'yes' );
				$result['success'] = true;
				break;

			/**
			 * The user response: Nope, maybe later
			 * Set next_favor_date in 7 days
			 */
			case 'later':
				emojics_set_option( 'next_review_favor', date('Y-m-d h:i:s', strtotime('+7 days')) );
				$result['success'] = true;
				break;

			/**
			 * The user response: Modal Close Button
			 * Set next_favor_date in 2 days
			 */
			case 'after_2_days':
				emojics_set_option( 'next_review_favor', date('Y-m-d h:i:s', strtotime('+2 days')) );
				$result['success'] = true;
				break;

			/**
			 * The user response: I already did
			 *
			 */
			case 'already':
				emojics_set_option( 'next_review_favor', 'no' );
				emojics_set_option( 'user_rated', 'yes' );
				$result['success'] = true;
				break;
		}

		echo json_encode( $result );
		die();
	}
}
