<?php

/**
 * The file that defines the core plugin class
 *
 * @link       http://www.emojics.com
 * @since      1.0.0
 *
 * @package    Emojics_WP
 * @subpackage Emojics_WP/includes
 * @author     Stefano <stefano.marra1987@gmail.com>
 */
class Emojics {
	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Emojics_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'emojics-wp';
		$this->version = EMJCS_VERSION;

		$this->load_dependencies();
		$this->set_locale();
		$this->plugin_setup();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		// Load Composer autoload if any
		// require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';

		/**
		 * Load plugin generic functions
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/functions-misc.php';

		/**
		 * Load plugin generic actions separated in tematic file names for easy reading
		 */
		foreach ( glob( plugin_dir_path( dirname( __FILE__ ) ) . 'includes/actions/*.php' ) as $filename ) {
			require_once $filename;
		}

		/**
		 * The class responsible for defining all the plugin variables like post types and metas
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-emojics-setup.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-emojics-loader.php';
		$this->loader = new Emojics_Loader();

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-emojics-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-emojics-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-emojics-public.php';

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Emojics_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Setup the plugin and add-ons
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function plugin_setup() {

		$plugin_setup = new Emojics_Setup( $this->get_plugin_name(), $this->get_version() );

		// Register Post Type
		// $this->loader->add_action( 'init', $plugin_setup, 'register_post_types' );

		// Register Settings Page
		$this->loader->add_action( 'admin_init', $plugin_setup, 'register_plugin_options' );
		$this->loader->add_action( 'admin_menu', $plugin_setup, 'register_plugin_pages' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Emojics_Admin( $this->get_plugin_name(), $this->get_version() );

		// Enqueue Scripts and Styles
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// Check from frontend ajax
		$this->loader->add_action( 'wp_ajax_emojics_api_authentication', $plugin_admin, 'authenticate_api_key_ajax' );
		$this->loader->add_action( 'wp_ajax_emojics_api_registration', $plugin_admin, 'registration_ajax' );
		$this->loader->add_action( 'wp_ajax_emojics_favor_response', $plugin_admin, 'handle_favor_response_ajax' );

		// Check when option update
		$this->loader->add_action( 'update_option_' . emojics_get_option_key('api_key'), $plugin_admin, 'authenticate_api_key_on_option_update', 10, 3 );

		// Remove some wp footer texts
		add_filter( 'admin_footer_text', '__return_empty_string', 11 );
		add_filter( 'update_footer', '__return_empty_string', 11 );

		// Show some admin messages
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'admin_messages' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Emojics_Public( $this->get_plugin_name(), $this->get_version() );

		// Enqueue Scripts and Styles
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		// Load emojics js code in footer and root tag after content if position = inline
		$this->loader->add_action( 'wp_footer', $plugin_public, 'load_js_code' );
		$this->loader->add_filter( 'the_content', $plugin_public, 'load_root_tag' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Emojics_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}
}
