<?php

/**
 * Gets the plugin specific option key
 */
function emojics_get_option_key( $option = null, $default = false ) {
	if ( is_null( $option ) ) return $default;

	return '_emojics_' . $option;
}

/**
 * Generic Option getter
 */
function emojics_get_option( $option = null, $default = false ) {
	if ( is_null( $option ) ) return $default;

	$option_value = get_option(emojics_get_option_key($option), $default);

	return $option_value;
}

/**
 * Generic Option setter
 */
function emojics_set_option( $option = null, $value ) {
	if ( is_null( $option ) ) return false;

	$result = update_option(emojics_get_option_key($option), $value);

	return $result;
}
/**
 * Generic Option delete
 */
function emojics_delete_option( $option = null ) {
	if ( is_null( $option ) ) return false;

	$result = delete_option(emojics_get_option_key($option));

	return $result;
}

/**
 * Get User IP
 *
 * Returns the IP address of the current visitor
 *
 */
function emojics_get_ip() {
	$ip = '127.0.0.1';
	if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
		//check ip from share internet
		$ip = $_SERVER['HTTP_CLIENT_IP'];
	} elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
		//to check ip is pass from proxy
		$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
	} elseif( ! empty( $_SERVER['REMOTE_ADDR'] ) ) {
		$ip = $_SERVER['REMOTE_ADDR'];
	}
	return apply_filters( 'emojics_get_ip', $ip );
}

/**
 * Returns the enviroment
 *
 * @return String
 * @since 1.0.0
 */
function emojics_get_env() {
	$user_ip = emojics_get_ip();

	$env = 'production';
	if ( substr($user_ip, 0, 4) == '127.' || $user_ip == '::1' ) {
		$env = 'development';
	}

	return apply_filters( 'emojics_get_enviroment', $env );
}

/**
 * Returns true if enviroment is development
 *
 * @return String
 * @since 1.0.0
 */
function emojics_is_env_dev() {

	$env = emojics_get_env();
	if ( $env == 'development' ) {
		return true;
	}

	return false;
}

/**
 * Returns true if enviroment is production
 *
 * @return String
 * @since 1.0.0
 */
function emojics_is_env_prod() {

	if ( ! emojics_is_env_dev() ) {
		return true;
	}

	return false;
}

/**
 * Returns an array with all the required registration fields
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_registration_fields() {
	$user_info = wp_get_current_user();

	return [
		'first_name' => $user_info->user_firstname,
		'last_name'  => $user_info->user_lastname,
		'site'       => get_home_url(),
		'email'      => $user_info->user_email,
		'password'   => null
	];
}

/**
 * Returns an array of supported type positions based on the passed type
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_initial_widget_configs() {
	$widget_configs = array();

	$excludes = (array)emojics_get_option('widget_exclude', array());
	$position = emojics_get_option('widget_position', 'floating');

	foreach (emojics_get_supported_types() as $key => $type) {
		$supported_positions = emojics_get_supported_type_position($key);

		$p = in_array($position, array_keys($supported_positions))?$position:'floating';

		switch ( $type->name ) {
			case 'post':
			case 'page':
				$p = 'inline';
				break;
		}

		$widget_configs[$key] = [
			'active'   => false,
			'position' => $p,
			'name'     => $type->name,
			'label'    => $type->label
		];
	}

	// $widget_configs['home']['active'] = (in_array('home', $excludes))?false:true;
	$widget_configs['post']['active'] = (in_array('post', $excludes))?false:true;
	$widget_configs['page']['active'] = (in_array('page', $excludes))?false:true;

	return $widget_configs;
}

/**
 * Returns an array of supported type positions based on the passed type
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_supported_type_position($type = null) {

	$only_floating = [
		'floating' => 'Floating'
	];

	$only_inline = [
		'inline' => 'Inline (after content)',
		'leftInline' => 'Inline Left (after content)',
		'rightInline' => 'Inline Right (after content)',
	];

	$both = array_merge($only_floating, $only_inline);

	if ( !$type ) {
		return [];
	}

	switch ($type) {
		case 'home':
		case 'page_not_found':
			return $only_floating;
			break;

		case 'author':
		case 'page':
			return $both;
			break;
	}

	$types = emojics_get_supported_types();

	if ( isset($types[$type]) && $types[$type]->hierarchical ) {
		return $only_floating;
	}

	return $both;
}

/**
 * Returns all pages like post-types, taxonomies, author pages
 * where the widget is supported
 *
 * Note: types refers to all kind of pages, also taxonomies and author pages
 *
 * @return Array
 * @since 1.2.0
 */
function emojics_get_supported_types() {

	$unsupported = ['post_format'];

	$types = [
		'home' => (object)[
			'name'         => 'home',
			'label'        => 'Home Page',
			'hierarchical' => false
		]
	];

	$types = array_merge( $types, get_post_types(array(
		'public'   => true,
		// '_builtin' => true
	), 'objects'));

	$types = array_merge( $types, [
		'author' => (object)[
			'name'         => 'author',
			'label'        => 'Author Pages',
			'hierarchical' => true
		]
	]);

	$types = array_merge( $types, get_taxonomies(array(
		'public'   => true,
		// '_builtin' => true
	), 'objects'));

	$types = array_merge( $types, [
		'page_not_found' => (object)[
			'name'         => 'page_not_found',
			'label'        => '404 Pages',
			'hierarchical' => false
		]
	]);

	/**
	 * Remove unsupported types
	 */
	foreach ($unsupported as $key) {
		if ( isset($types[$key]) ) {
			unset($types[$key]);
		}
	}

	return apply_filters( 'emojics_get_post_types', $types);
}

/**
 * Return the system info
 */
function emojics_get_system_info() {
	global $wpdb;

	if( !class_exists( 'Browser' ) )
		require_once 'browser.php';

	$browser = new Browser();

	// Get theme info
	$theme_data = wp_get_theme();
	$theme      = $theme_data->Name . ' ' . $theme_data->Version;

	// Try to identify the hosting provider
	$host = emojics_get_host();

	$return  = '### Begin System Info ###' . "\n\n";

	// Start with the basics...
	$return .= '////-- Site Info' . "\n\n";
	$return .= 'Site URL:                 ' . site_url() . "\n";
	$return .= 'Home URL:                 ' . home_url() . "\n";
	$return .= 'Multisite:                ' . ( is_multisite() ? 'Yes' : 'No' ) . "\n";

	$return  = apply_filters( 'emojics_sysinfo_after_site_info', $return );

	// Can we determine the site's host?
	if( $host ) {
		$return .= "\n" . '////-- Hosting Provider' . "\n\n";
		$return .= 'Host:                     ' . $host . "\n";

		$return  = apply_filters( 'emojics_sysinfo_after_host_info', $return );
	}

	// The local users' browser information, handled by the Browser class
	$return .= "\n" . '////-- User Browser' . "\n\n";
	$return .= $browser;

	$return  = apply_filters( 'emojics_sysinfo_after_user_browser', $return );

	// WordPress configuration
	$return .= "\n" . '////-- WordPress Configuration' . "\n\n";

	$return .= 'Version:                  ' . get_bloginfo( 'version' ) . "\n";
	$return .= 'Language:                 ' . ( defined( 'WPLANG' ) && WPLANG ? WPLANG : 'en_US' ) . "\n";
	$return .= 'Permalink Structure:      ' . ( get_option( 'permalink_structure' ) ? get_option( 'permalink_structure' ) : 'Default' ) . "\n";
	$return .= 'Active Theme:             ' . $theme . "\n";
	$return .= 'Show On Front:            ' . get_option( 'show_on_front' ) . "\n";

	// Only show page specs if frontpage is set to 'page'
	if( get_option( 'show_on_front' ) == 'page' ) {
		$front_page_id = get_option( 'page_on_front' );
		$blog_page_id = get_option( 'page_for_posts' );

		$return .= 'Page On Front:            ' . ( $front_page_id != 0 ? get_the_title( $front_page_id ) . ' (#' . $front_page_id . ')' : 'Unset' ) . "\n";
		$return .= 'Page For Posts:           ' . ( $blog_page_id != 0 ? get_the_title( $blog_page_id ) . ' (#' . $blog_page_id . ')' : 'Unset' ) . "\n";
	}

	// Make sure wp_remote_post() is working
	$request['cmd'] = '_notify-validate';

	$params = array(
		'sslverify'     => false,
		'timeout'       => 60,
		'user-agent'    => 'emojics/',
		'body'          => $request
	);

	$response = wp_remote_post( 'https://www.paypal.com/cgi-bin/webscr', $params );

	if( !is_wp_error( $response ) && $response['response']['code'] >= 200 && $response['response']['code'] < 300 ) {
		$WP_REMOTE_POST = 'wp_remote_post() works';
	} else {
		$WP_REMOTE_POST = 'wp_remote_post() does not work';
	}

	$return .= 'Remote Post:              ' . $WP_REMOTE_POST . "\n";
	$return .= 'Table Prefix:             ' . 'Length: ' . strlen( $wpdb->prefix ) . '   Status: ' . ( strlen( $wpdb->prefix ) > 16 ? 'ERROR: Too long' : 'Acceptable' ) . "\n";
	// Commented out per https://github.com/easydigitaldownloads/Easy-Digital-Downloads/issues/3475
	//$return .= 'Admin AJAX:               ' . ( emojics_test_ajax_works() ? 'Accessible' : 'Inaccessible' ) . "\n";
	$return .= 'WP_DEBUG:                 ' . ( defined( 'WP_DEBUG' ) ? WP_DEBUG ? 'Enabled' : 'Disabled' : 'Not set' ) . "\n";
	$return .= 'Memory Limit:             ' . WP_MEMORY_LIMIT . "\n";
	$return .= 'Registered Post Status:   ' . implode( ', ', get_post_stati() ) . "\n";

	$return  = apply_filters( 'emojics_sysinfo_after_wordpress_config', $return );

	// emojics configuration database settings
	$config_opt  = get_option('_emojics_widget_configs');
	$api_key_opt  = get_option('_emojics_api_key');
	$js_code_opt  = get_option('_emojics_js_code');
	$is_widget_active = get_option('_emojics_widget_active');
	$is_debug_active  = get_option('_emojics_widget_debug');

	$return .= "\n" . '////-- Emojics WP Plugin Configuration' . "\n\n";
	$return .= 'Version:                    ' . EMJCS_VERSION . "\n";
	$return .= 'Configuration:   ' . "\n";
	foreach ($config_opt as $key => $value) {
		$return .= '   ' . $key . ':					' . json_encode($value) . "\n";
	}
	$return .= "\n";
	$return .= 'Is Active:					' . $is_widget_active . "\n";
	$return .= 'Is Debug Active:			' . $is_debug_active . "\n";
	$return .= 'API Key:					' . $api_key_opt . "\n";
	$return .= 'JS Code:					' . "\n" . $js_code_opt . "\n";

	// Get plugins that have an update
	$updates = get_plugin_updates();

	// Must-use plugins
	// NOTE: MU plugins can't show updates!
	$muplugins = get_mu_plugins();
	if( count( $muplugins > 0 ) ) {
		$return .= "\n" . '////-- Must-Use Plugins' . "\n\n";

		foreach( $muplugins as $plugin => $plugin_data ) {
			$return .= $plugin_data['Name'] . ': ' . $plugin_data['Version'] . "\n";
		}

		$return = apply_filters( 'emojics_sysinfo_after_wordpress_mu_plugins', $return );
	}

	// WordPress active plugins
	$return .= "\n" . '////-- WordPress Active Plugins' . "\n\n";

	$plugins = get_plugins();
	$active_plugins = get_option( 'active_plugins', array() );

	foreach( $plugins as $plugin_path => $plugin ) {
		if( !in_array( $plugin_path, $active_plugins ) )
			continue;

		$update = ( array_key_exists( $plugin_path, $updates ) ) ? ' (needs update - ' . $updates[$plugin_path]->update->new_version . ')' : '';
		$return .= $plugin['Name'] . ': ' . $plugin['Version'] . $update . "\n";
	}

	$return  = apply_filters( 'emojics_sysinfo_after_wordpress_plugins', $return );

	// WordPress inactive plugins
	$return .= "\n" . '////-- WordPress Inactive Plugins' . "\n\n";

	foreach( $plugins as $plugin_path => $plugin ) {
		if( in_array( $plugin_path, $active_plugins ) )
			continue;

		$update = ( array_key_exists( $plugin_path, $updates ) ) ? ' (needs update - ' . $updates[$plugin_path]->update->new_version . ')' : '';
		$return .= $plugin['Name'] . ': ' . $plugin['Version'] . $update . "\n";
	}

	$return  = apply_filters( 'emojics_sysinfo_after_wordpress_plugins_inactive', $return );

	if( is_multisite() ) {
		// WordPress Multisite active plugins
		$return .= "\n" . '////-- Network Active Plugins' . "\n\n";

		$plugins = wp_get_active_network_plugins();
		$active_plugins = get_site_option( 'active_sitewide_plugins', array() );

		foreach( $plugins as $plugin_path ) {
			$plugin_base = plugin_basename( $plugin_path );

			if( !array_key_exists( $plugin_base, $active_plugins ) )
				continue;

			$update = ( array_key_exists( $plugin_path, $updates ) ) ? ' (needs update - ' . $updates[$plugin_path]->update->new_version . ')' : '';
			$plugin  = get_plugin_data( $plugin_path );
			$return .= $plugin['Name'] . ': ' . $plugin['Version'] . $update . "\n";
		}

		$return  = apply_filters( 'emojics_sysinfo_after_wordpress_ms_plugins', $return );
	}

	// Server configuration (really just versioning)
	$return .= "\n" . '////-- Webserver Configuration' . "\n\n";
	$return .= 'PHP Version:              ' . PHP_VERSION . "\n";
	$return .= 'MySQL Version:            ' . $wpdb->db_version() . "\n";
	$return .= 'Webserver Info:           ' . $_SERVER['SERVER_SOFTWARE'] . "\n";

	$return  = apply_filters( 'emojics_sysinfo_after_webserver_config', $return );

	// PHP configs... now we're getting to the important stuff
	$return .= "\n" . '////-- PHP Configuration' . "\n\n";
	$return .= 'Safe Mode:                ' . ( ini_get( 'safe_mode' ) ? 'Enabled' : 'Disabled' . "\n" );
	$return .= 'Memory Limit:             ' . ini_get( 'memory_limit' ) . "\n";
	$return .= 'Upload Max Size:          ' . ini_get( 'upload_max_filesize' ) . "\n";
	$return .= 'Post Max Size:            ' . ini_get( 'post_max_size' ) . "\n";
	$return .= 'Upload Max Filesize:      ' . ini_get( 'upload_max_filesize' ) . "\n";
	$return .= 'Time Limit:               ' . ini_get( 'max_execution_time' ) . "\n";
	$return .= 'Max Input Vars:           ' . ini_get( 'max_input_vars' ) . "\n";
	$return .= 'Display Errors:           ' . ( ini_get( 'display_errors' ) ? 'On (' . ini_get( 'display_errors' ) . ')' : 'N/A' ) . "\n";

	$return  = apply_filters( 'emojics_sysinfo_after_php_config', $return );

	// PHP extensions and such
	$return .= "\n" . '////-- PHP Extensions' . "\n\n";
	$return .= 'cURL:                     ' . ( function_exists( 'curl_init' ) ? 'Supported' : 'Not Supported' ) . "\n";
	$return .= 'fsockopen:                ' . ( function_exists( 'fsockopen' ) ? 'Supported' : 'Not Supported' ) . "\n";
	$return .= 'SOAP Client:              ' . ( class_exists( 'SoapClient' ) ? 'Installed' : 'Not Installed' ) . "\n";
	$return .= 'Suhosin:                  ' . ( extension_loaded( 'suhosin' ) ? 'Installed' : 'Not Installed' ) . "\n";
	$return .= 'Mbstring:                 ' . ( extension_loaded( 'mbstring' ) ? 'Installed' : 'Not Installed' ) . "\n";

	$return  = apply_filters( 'emojics_sysinfo_after_php_ext', $return );

	// Session stuff
	$return .= "\n" . '////-- Session Configuration' . "\n\n";
	$return .= 'Session:                  ' . ( isset( $_SESSION ) ? 'Enabled' : 'Disabled' ) . "\n";

	// The rest of this is only relevant is session is enabled
	if( isset( $_SESSION ) ) {
		$return .= 'Session Name:             ' . esc_html( ini_get( 'session.name' ) ) . "\n";
		$return .= 'Cookie Path:              ' . esc_html( ini_get( 'session.cookie_path' ) ) . "\n";
		$return .= 'Save Path:                ' . esc_html( ini_get( 'session.save_path' ) ) . "\n";
		$return .= 'Use Cookies:              ' . ( ini_get( 'session.use_cookies' ) ? 'On' : 'Off' ) . "\n";
		$return .= 'Use Only Cookies:         ' . ( ini_get( 'session.use_only_cookies' ) ? 'On' : 'Off' ) . "\n";
	}

	$return  = apply_filters( 'emojics_sysinfo_after_session_config', $return );

	$return .= "\n" . '### End System Info ###';

	return $return;
}

/**
 * Returns the webhost the site is using if possible, false otherwise
 */
function emojics_get_host() {
	$host = false;

	if ( defined( 'WPE_APIKEY' ) ) {
		$host = 'WP Engine';
	} elseif( defined( 'PAGELYBIN' ) ) {
		$host = 'Pagely';
	} elseif( DB_HOST == 'localhost:/tmp/mysql5.sock' ) {
		$host = 'ICDSoft';
	} elseif( DB_HOST == 'mysqlv5' ) {
		$host = 'NetworkSolutions';
	} elseif( strpos( DB_HOST, 'ipagemysql.com' ) !== false ) {
		$host = 'iPage';
	} elseif( strpos( DB_HOST, 'ipowermysql.com' ) !== false ) {
		$host = 'IPower';
	} elseif( strpos( DB_HOST, '.gridserver.com' ) !== false ) {
		$host = 'MediaTemple Grid';
	} elseif( strpos( DB_HOST, '.pair.com' ) !== false ) {
		$host = 'pair Networks';
	} elseif( strpos( DB_HOST, '.stabletransit.com' ) !== false ) {
		$host = 'Rackspace Cloud';
	} elseif( strpos( DB_HOST, '.sysfix.eu' ) !== false ) {
		$host = 'SysFix.eu Power Hosting';
	} elseif( strpos( $_SERVER['SERVER_NAME'], 'Flywheel' ) !== false ) {
		$host = 'Flywheel';
	} else {
		// Adding a general fallback for data gathering
		$host = 'DBH: ' . DB_HOST . ', SRV: ' . $_SERVER['SERVER_NAME'];
	}

	return $host;
}

/**
 * Return metadata
 */
function emojics_get_metadata() {

	$metadata = [];
	$metadata_keys = emojics_get_option('widget_metadata', []);

	if ( is_user_logged_in() && $metadata_keys && is_array($metadata_keys) ) {

		$user = wp_get_current_user();
		foreach ($metadata_keys as $meta_key) {

			switch ($meta_key) {
				case 'user_email':
					$metadata['$email'] = $user->{$meta_key};
					break;

				case 'display_name':
					$metadata['$name'] = $user->{$meta_key};
					break;

				case 'ID':
					$metadata['user_id'] = $user->{$meta_key};
					break;
			}
		}
	}

	return apply_filters( 'emojics_metadata', $metadata );
}

/**
 * Returns true if the position is inline
 *
 * @return Boolean
 */
function emojics_is_position_inline($position = null) {

	switch ($position) {
		case 'inline':
		case 'leftInline':
		case 'rightInline':
			return true;
	}

	return false;
}