<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://www.emojics.com
 * @since      1.0.0
 *
 * @package    Emojics
 * @subpackage Emojics/public
 * @author     Stefano <stefano.marra1987@gmail.com>
 */
class Emojics_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {}

	/**
	 * Function returns true if the current page/post is excluded
	 *
	 * @since 	1.1.0
	 */
	private function is_excluded($post_id = null) {
		if ( is_null($post_id) ) {
			$post_id = get_the_ID();
		}

		if ( !$post_id ) {
			return false;
		}

		$excludes = (array)emojics_get_option('widget_exclude', array());

		/**
		 * Homepage excluded
		 */
		$home_excluded = in_array('home', $excludes);
		if ( $home_excluded && (is_front_page() || is_home()) ) {
			return true;
		}

		/**
		 * Post type excluded
		 */
		if ( in_array(get_post_type( $post_id ), $excludes) ) {
			return true;
		}

		// var_dump(is_home());die();

		return false;
	}

	/**
	 * Function return true if post/page/category is enabled
	 *
	 * @since 1.2.0
	 */
	function get_enabled_widget_config($post_id = null) {
		if ( is_null($post_id) ) {
			$post_id = get_the_ID();
		}

		$widget_configs = emojics_get_option('widget_configs', emojics_get_initial_widget_configs());

		/**
		 * Home Page Check
		 */
		if ( is_front_page() || is_home() ) {
			if ( isset($widget_configs['home']) ) {
				return ($widget_configs['home']['active']=='yes')?$widget_configs['home']:false;
			}
			return false;
		}

		/**
		 * Taxonomy Check
		 */
		if ( is_archive() ) {
			$taxonomy = get_queried_object()->taxonomy;
			if ( $taxonomy && isset($widget_configs[$taxonomy]) ) {
				return ($widget_configs[$taxonomy]['active']=='yes')?$widget_configs[$taxonomy]:false;
			}
			return false;
		}

		/**
		 * Author Check
		 */
		if ( is_author() ) {
			if ( isset($widget_configs['author']) ) {
				return ($widget_configs['author']['active']=='yes')?$widget_configs['author']:false;
			}
			return false;
		}

		/**
		 * Post Type / Custom Post Type Check
		 */
		$post_type = get_post_type( $post_id );
		if ( isset($widget_configs[$post_type]) ) {
			return ($widget_configs[$post_type]['active']=='yes')?$widget_configs[$post_type]:false;
		}

		/**
		 * 404 Check
		 */
		if ( is_404() ) {
			if ( isset($widget_configs['page_not_found']) ) {
				return ($widget_configs['page_not_found']['active']=='yes')?$widget_configs['page_not_found']:false;
			}
			return false;
		}

		return false;
	}


	/**
	 * Load emojics js code
	 *
	 * @since    1.0.0
	 */
	public function load_js_code() {

		$js_code = emojics_get_option('js_code', false);
		$debug = emojics_get_option('widget_debug', 'no')=='yes'?true:false;
		$metadata = emojics_get_metadata();

		if ( !$js_code ) {
			return false;
		}

		if ( !$config = $this->get_enabled_widget_config() ) {
			return false;
		}

		/**
		 * If position is inline:
		 * - Remove <div id="emojics-root"></div> from the js code
		 * - Add position inline param
		 */
		if ( emojics_is_position_inline($config['position']) ) {

			if ( strpos($js_code, EMJCS_ROOT_TAG) ) {

				/**
				 * Remove emojics root tag
				 */
				$js_code = str_replace(EMJCS_ROOT_TAG, '', $js_code);

				/**
				 * Add inline position param
				 */
				$pattern = "/^.*\bwidget\b.*$/m";
				preg_match($pattern, $js_code, $matches);
				if ( isset($matches[0]) && !empty($matches[0]) ) {
					$js_code = str_replace($matches[0], $matches[0] . ', position: \'' . $config['position'] .  '\'', $js_code);
				}
			}
		}

		/**
		 * Add debug param if set
		 */
		if ( $debug ) {
			$pattern = "/^.*\bwidget\b.*$/m";
			preg_match($pattern, $js_code, $matches);
			if ( isset($matches[0]) && !empty($matches[0]) ) {
				$js_code = str_replace($matches[0], $matches[0] . ', debug: true', $js_code);
			}
		}

		/**
		 * Add metadata param if set
		 */
		if ( $metadata && is_array($metadata) ) {
			$pattern = "/^.*\bwidget\b.*$/m";
			preg_match($pattern, $js_code, $matches);
			if ( isset($matches[0]) && !empty($matches[0]) ) {
				$js_code = str_replace($matches[0], $matches[0] . ', metadata: ' . json_encode($metadata), $js_code);
			}
		}

		echo $js_code;
	}

	/**
	 * Load emojics root tag
	 *
	 * @since    1.1.0
	 */
	public function load_root_tag($content) {

		if ( !$config = $this->get_enabled_widget_config() ) {
			return $content;
		}

		if ( !emojics_is_position_inline($config['position']) ) {
			return $content;
		}

		// if ( !is_singular() ) {
		// 	return $content;
		// }

		/**
		 * From here on, the widget position is inline
		 */

		return $content . EMJCS_ROOT_TAG;
	}
}
